package cn.com.poc.long_document.aggregate.impl;

import cn.com.poc.agent_application.entity.BizAgentApplicationGcConfigEntity;
import cn.com.poc.agent_application.entity.KnowledgeContentResult;
import cn.com.poc.agent_application.service.BizAgentApplicationGcConfigService;
import cn.com.poc.common.constant.CommonConstant;
import cn.com.poc.common.model.BizFileUploadRecordModel;
import cn.com.poc.common.service.BizFileUploadRecordService;
import cn.com.poc.common.utils.*;
import cn.com.poc.knowledge.aggregate.KnowledgeService;
import cn.com.poc.knowledge.constant.KnowledgeConstant;
import cn.com.poc.knowledge.entity.BizKnowledgeDocumentEntity;
import cn.com.poc.knowledge.query.KnowledgeDocumentRelationQueryItem;
import cn.com.poc.knowledge.service.BizKnowledgeDocumentService;
import cn.com.poc.knowledge.service.BizKnowledgeInfoService;
import cn.com.poc.long_document.aggregate.LongTextDialoguesService;
import cn.com.poc.long_document.domain.LongtextDialoguesResult;
import cn.com.poc.long_document.dto.DialoguesContextDto;
import cn.com.poc.long_document.dto.LongTextDialoguesDto;
import cn.com.poc.long_document.dto.LongTextExampleDto;
import cn.com.poc.long_document.model.BizLongTextDialoguesModel;
import cn.com.poc.long_document.model.BizLongTextDialoguesRecordModel;
import cn.com.poc.long_document.model.BizLongTextExampleModel;
import cn.com.poc.long_document.service.BizLongTextDialoguesRecordService;
import cn.com.poc.long_document.service.BizLongTextDialoguesService;
import cn.com.poc.long_document.service.BizLongTextExampleService;
import cn.com.poc.support.security.oauth.entity.UserBaseEntity;
import cn.com.poc.thirdparty.resource.demand.ai.aggregate.DemandKnowledgeService;
import cn.com.poc.thirdparty.resource.demand.ai.constants.KnowledgeSearchTypeEnum;
import cn.com.poc.thirdparty.resource.demand.ai.constants.LLMRoleEnum;
import cn.com.poc.thirdparty.resource.demand.ai.entity.dialogue.Message;
import cn.com.poc.thirdparty.resource.demand.ai.entity.knowledge.SearchKnowledgeResult;
import cn.com.poc.thirdparty.resource.demand.ai.entity.largemodel.LargeModelDemandResult;
import cn.com.poc.thirdparty.resource.demand.ai.entity.largemodel.LargeModelResponse;
import cn.com.poc.thirdparty.service.LLMService;
import cn.com.yict.framemax.core.exception.BusinessException;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.lang.ArrayUtils;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.io.BufferedReader;
import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.stream.Collectors;

/**
 * @author alex.yao
 * @date 2025/6/17
 */
@Service
public class LongTextDialoguesServiceImpl implements LongTextDialoguesService {

    private final Logger logger = LoggerFactory.getLogger(LongTextDialoguesService.class);

    @Resource
    private LLMService llmService;

    @Resource
    private BizFileUploadRecordService bizFileUploadRecordService;

    @Resource
    private BizLongTextDialoguesService bizLongTextDialoguesService;

    @Resource
    private BizLongTextDialoguesRecordService bizLongTextDialoguesRecordService;

    @Resource
    private BizLongTextExampleService bizLongTextExampleService;

    @Resource
    private BizAgentApplicationGcConfigService bizAgentApplicationGcConfigService;

    @Resource
    private KnowledgeService knowledgeService;

    @Resource
    private BizKnowledgeDocumentService bizKnowledgeDocumentService;

    @Resource
    private DemandKnowledgeService demandKnowledgeService;


    @Override
    public String create(String fileUrl, Long userId) throws Exception {
        String dialoguesId = UUIDTool.getUUID();
        BizLongTextDialoguesModel model = new BizLongTextDialoguesModel();
        model.setDialogsId(dialoguesId);
        model.setMemberId(userId);
        model.setFileUrl(fileUrl);
        model.setIsDeleted(CommonConstant.IsDeleted.N);
        bizLongTextDialoguesService.save(model);
        return dialoguesId;
    }

    @Override
    public void call(String dialoguesId, String fileUrl, String input, Integer[] knowledgeIds, Long userId) throws Exception {
        UserBaseEntity userBaseEntity = BlContext.getCurrentUserNotException();
        if (userBaseEntity == null) {
            throw new BusinessException("用户未登录");
        }

        BizLongTextDialoguesModel model = new BizLongTextDialoguesModel();
        model.setDialogsId(dialoguesId);
        model.setMemberId(userBaseEntity.getUserId());
        model.setIsDeleted(CommonConstant.IsDeleted.N);
        List<BizLongTextDialoguesModel> models = bizLongTextDialoguesService.findByExample(model, null);

        if (CollectionUtils.isEmpty(models)) {
            throw new BusinessException("对话ID不存在");
        }

        BizLongTextDialoguesModel bizLongTextDialoguesModel = models.get(0);

        //更新标题
        if (StringUtils.isBlank(bizLongTextDialoguesModel.getTitle())) { // 标题
            bizLongTextDialoguesModel.setTitle(input.substring(0, Math.min(input.length(), 150)));
            bizLongTextDialoguesModel = bizLongTextDialoguesService.save(bizLongTextDialoguesModel);
        }
        BizLongTextDialoguesRecordModel dialoguesRecordModel = new BizLongTextDialoguesRecordModel();
        dialoguesRecordModel.setDialogsId(dialoguesId);
        dialoguesRecordModel.setMemberId(userBaseEntity.getUserId());
        dialoguesRecordModel.setIsDeleted(CommonConstant.IsDeleted.N);
        List<BizLongTextDialoguesRecordModel> dialoguesRecordModelList = bizLongTextDialoguesRecordService.findByExample(dialoguesRecordModel, null);

        boolean isInit = false;
        if (CollectionUtils.isEmpty(dialoguesRecordModelList)) {
            if (StringUtils.isNoneBlank(bizLongTextDialoguesModel.getFileUrl()) && StringUtils.isBlank(fileUrl)) {
                fileUrl = bizLongTextDialoguesModel.getFileUrl();
            }
            isInit = true;
        }

        // 文件是否变更
        boolean isFileUrlChanged = (StringUtils.isNotBlank(fileUrl) && !bizLongTextDialoguesModel.getFileUrl().equals(fileUrl))
                || isInit;

        if (StringUtils.isBlank(fileUrl)) {
            fileUrl = bizLongTextDialoguesModel.getFileUrl();
        }

        // 更新文件URL
        if (isFileUrlChanged) {
            if (!isInit && StringUtils.isNotBlank(fileUrl)) {
                bizLongTextDialoguesModel.setFileUrl(fileUrl);
                bizLongTextDialoguesService.save(bizLongTextDialoguesModel);
            }
        }

        //记录提问时间
        long inputTimestamp = System.currentTimeMillis();

        SSEUtil sseUtil = new SSEUtil();
        // 知识库
        //获取知识库配置
        List<Integer> kdIds = knowledgeService.getKdIdsByKnowledgeInfoIds(knowledgeIds);
        List<KnowledgeContentResult> knowledgeResult = knowledge(kdIds, input, 5, 0.5, KnowledgeSearchTypeEnum.MIX);

        if (CollectionUtils.isNotEmpty(knowledgeResult)) {
            LargeModelDemandResult result = new LargeModelDemandResult();
            result.setCode("0");
            result.setFunction(null);
            result.setDbChainResult(null);
            result.setKnowledgeContentResult(knowledgeResult);
            sseUtil.send(JsonUtils.serialize(result));
        }


        //组装请求参数
        List<Message> messages = buildMessages(dialoguesId, userBaseEntity.getUserId(), input, fileUrl, knowledgeResult);
        LargeModelResponse largeModelResponse = new LargeModelResponse();
        largeModelResponse.setModel("qwen-long");
        largeModelResponse.setMessages(messages.toArray(new Message[0]));
        largeModelResponse.setStream(true);
        BufferedReader bufferedReader = llmService.chatChunk(largeModelResponse);
        LongtextDialoguesResult longtextDialoguesResult = textOutputStream(sseUtil, bufferedReader);


        // 保存用户输入记录
        BizLongTextDialoguesRecordModel inputRecord = new BizLongTextDialoguesRecordModel();
        inputRecord.setMemberId(userBaseEntity.getUserId());
        inputRecord.setContent(input);
        if (isFileUrlChanged) {
            inputRecord.setFileUrl(fileUrl);
        }
        inputRecord.setDialogsId(dialoguesId);
        inputRecord.setRole("user");
        inputRecord.setTimestamp(inputTimestamp);

        // 保存AI回复记录
        BizLongTextDialoguesRecordModel assistantRecord = new BizLongTextDialoguesRecordModel();
        assistantRecord.setMemberId(userBaseEntity.getUserId());
        assistantRecord.setContent(longtextDialoguesResult.getMessage());
        assistantRecord.setReasoningContent(longtextDialoguesResult.getReasoningContent());
        assistantRecord.setDialogsId(dialoguesId);
        assistantRecord.setRole("assistant");
        assistantRecord.setTimestamp(System.currentTimeMillis());
        List<BizLongTextDialoguesRecordModel> recordModels = new ArrayList<>();
        recordModels.add(inputRecord);
        recordModels.add(assistantRecord);
        bizLongTextDialoguesRecordService.saveAll(recordModels);
    }

    @Override
    public void delete(String dialoguesId, Long userId) throws Exception {
        BizLongTextDialoguesModel model = new BizLongTextDialoguesModel();
        model.setDialogsId(dialoguesId);
        model.setMemberId(userId);
        model.setIsDeleted(CommonConstant.IsDeleted.N);
        List<BizLongTextDialoguesModel> models = bizLongTextDialoguesService.findByExample(model, null);
        if (CollectionUtils.isEmpty(models)) {
            throw new BusinessException("对话ID不存在");
        }
        BizLongTextDialoguesModel bizLongTextDialoguesModel = models.get(0);
        bizLongTextDialoguesModel.setIsDeleted(CommonConstant.IsDeleted.Y);
        bizLongTextDialoguesService.save(bizLongTextDialoguesModel);
        logger.info("删除对话ID:{}", dialoguesId);
    }

    @Override
    public List<LongTextDialoguesDto> dialoguesList(Long userId) throws Exception {
        List<LongTextDialoguesDto> dialoguesList = new ArrayList<>();
        BizLongTextDialoguesModel model = new BizLongTextDialoguesModel();
        model.setMemberId(userId);
        model.setIsDeleted(CommonConstant.IsDeleted.N);
        List<BizLongTextDialoguesModel> models = bizLongTextDialoguesService.findByExample(model, null);
        if (CollectionUtils.isNotEmpty(models)) {
            for (BizLongTextDialoguesModel bizLongTextDialoguesModel : models) {
                if (StringUtils.isBlank(bizLongTextDialoguesModel.getTitle())) {
                    continue;
                }
                LongTextDialoguesDto dto = new LongTextDialoguesDto();
                dto.setDialogsId(bizLongTextDialoguesModel.getDialogsId());
                dto.setTitle(bizLongTextDialoguesModel.getTitle());
                dto.setFileUrl(bizLongTextDialoguesModel.getFileUrl());
                if (StringUtils.isNotBlank(bizLongTextDialoguesModel.getFileUrl())) {
                    BizFileUploadRecordModel bizFileUploadRecordModel = new BizFileUploadRecordModel();
                    bizFileUploadRecordModel.setFileUrl(bizLongTextDialoguesModel.getFileUrl());
                    List<BizFileUploadRecordModel> fileUploadRecordModels = bizFileUploadRecordService.findByExample(bizFileUploadRecordModel, null);
                    if (CollectionUtils.isNotEmpty(fileUploadRecordModels)) {
                        dto.setFileName(fileUploadRecordModels.get(0).getFileName());
                    }
                }
                dto.setMemberId(userId);
                dto.setCreatedTime(bizLongTextDialoguesModel.getCreatedTime());
                dialoguesList.add(dto);
            }
        }
        return dialoguesList;
    }

    @Override
    public List<DialoguesContextDto> dialoguesContext(String dialoguesId, Long userId) throws Exception {
        BizLongTextDialoguesModel model = new BizLongTextDialoguesModel();
        model.setMemberId(userId);
        model.setDialogsId(dialoguesId);
        List<BizLongTextDialoguesModel> models = bizLongTextDialoguesService.findByExample(model, null);
        if (CollectionUtils.isEmpty(models)) {
            throw new BusinessException("对话ID不存在");
        }

        BizLongTextDialoguesRecordModel recordModel = new BizLongTextDialoguesRecordModel();
        recordModel.setDialogsId(dialoguesId);
        recordModel.setMemberId(model.getMemberId());
        recordModel.setIsDeleted(CommonConstant.IsDeleted.N);
        List<BizLongTextDialoguesRecordModel> recordModels = bizLongTextDialoguesRecordService.findByExample(recordModel, null);
        List<DialoguesContextDto> dialoguesContextDtos = new ArrayList<>();
        if (CollectionUtils.isNotEmpty(recordModels)) {
            for (BizLongTextDialoguesRecordModel dialoguesRecordModel : recordModels) {
                DialoguesContextDto dialoguesContextDto = new DialoguesContextDto();
                dialoguesContextDto.setRole(dialoguesRecordModel.getRole());
                dialoguesContextDto.setContent(dialoguesRecordModel.getContent());
                dialoguesContextDto.setReasoningContent(dialoguesRecordModel.getReasoningContent());
                dialoguesContextDto.setTimestamp(dialoguesRecordModel.getTimestamp());
                dialoguesContextDto.setFileUrl(dialoguesRecordModel.getFileUrl());
                if (StringUtils.isNotBlank(dialoguesRecordModel.getFileUrl())) {
                    BizFileUploadRecordModel bizFileUploadRecordModel = new BizFileUploadRecordModel();
                    bizFileUploadRecordModel.setFileUrl(dialoguesRecordModel.getFileUrl());
                    List<BizFileUploadRecordModel> fileUploadRecordModels = bizFileUploadRecordService.findByExample(bizFileUploadRecordModel, null);
                    if (CollectionUtils.isNotEmpty(fileUploadRecordModels)) {
                        dialoguesContextDto.setFileName(fileUploadRecordModels.get(0).getFileName());
                    }
                }
                dialoguesContextDtos.add(dialoguesContextDto);
            }
        }
        return dialoguesContextDtos;
    }

    @Override
    public List<LongTextExampleDto> example() throws Exception {
        List<LongTextExampleDto> result = new ArrayList<>();
        BizLongTextExampleModel bizLongTextExampleModel = new BizLongTextExampleModel();
        bizLongTextExampleModel.setIsDeleted(CommonConstant.IsDeleted.N);
        Collection<BizLongTextExampleModel> models = bizLongTextExampleService.findByExample(bizLongTextExampleModel, null);
        if (CollectionUtils.isNotEmpty(models)) {
            for (BizLongTextExampleModel model : models) {
                LongTextExampleDto dto = new LongTextExampleDto();
                dto.setTitle(model.getTitle());
                dto.setFileUrl(model.getFileUrl());
                dto.setPreviewImage(model.getPreviewImage());
                if (StringUtils.isNotBlank(model.getFileUrl())) {
                    BizFileUploadRecordModel bizFileUploadRecordModel = new BizFileUploadRecordModel();
                    bizFileUploadRecordModel.setFileUrl(model.getFileUrl());
                    List<BizFileUploadRecordModel> fileUploadRecordModels = bizFileUploadRecordService.findByExample(bizFileUploadRecordModel, null);
                    if (CollectionUtils.isNotEmpty(fileUploadRecordModels)) {
                        dto.setFileName(fileUploadRecordModels.get(0).getFileName());
                    }
                }
                dto.setDescription(model.getDescription());
                dto.setQuestion(model.getQuestion());
                result.add(dto);
            }
        }
        return result;
    }


    private List<Message> buildMessages(String dialogsId, Long userId, String input, String fileUrl, List<KnowledgeContentResult> knowledgeContentResults) throws Exception {
        // 获取对话提示词
        String promptCode = "DocumentDialoguePrompt";
        BizAgentApplicationGcConfigEntity documentDialoguePrompt = bizAgentApplicationGcConfigService.getByConfigCode(promptCode);
        if (documentDialoguePrompt == null || StringUtils.isBlank(documentDialoguePrompt.getConfigSystem())) {
            logger.error("获取对话提示词失败 , configCode:{}", promptCode);
            throw new BusinessException("获取对话提示词失败");
        }

        // 加载文件
        File file = DocumentLoad.downloadURLDocument(fileUrl);
        String fileContent = DocumentLoad.documentToText(file);


        // 加载对话提示词
        String prompt = documentDialoguePrompt.getConfigSystem();
        if (CollectionUtils.isNotEmpty(knowledgeContentResults)) {
            String knowledgeContent = JsonUtils.serialize(knowledgeContentResults);
            prompt = prompt.replace("${knowledgeContent}", knowledgeContent);
        } else {
            prompt = prompt.replace("${knowledgeContent}", StringUtils.EMPTY);
        }

        // 配置message
        List<Message> messages = new ArrayList<>();
        Message systemMessage = new Message();
        systemMessage.setContent(prompt);
        systemMessage.setRole(LLMRoleEnum.SYSTEM.getRole());
        messages.add(systemMessage);

        // 文件内容
        Message fileSystemMessage = new Message();
        fileSystemMessage.setContent(fileContent);
        fileSystemMessage.setRole(LLMRoleEnum.SYSTEM.getRole());
        messages.add(fileSystemMessage);

        // 对话记录
        BizLongTextDialoguesRecordModel recordModel = new BizLongTextDialoguesRecordModel();
        recordModel.setDialogsId(dialogsId);
        recordModel.setIsDeleted(CommonConstant.IsDeleted.N);
        recordModel.setMemberId(userId);
        List<BizLongTextDialoguesRecordModel> recordModels = bizLongTextDialoguesRecordService.findByExample(recordModel, null);
        if (CollectionUtils.isNotEmpty(recordModels)) {
            for (BizLongTextDialoguesRecordModel entity : recordModels) {
                Message message = new Message();
                message.setContent(entity.getContent());
                message.setRole(entity.getRole());
                messages.add(message);
            }
        }
        //判断最后是否为User，若是则删除。
        if (CollectionUtils.isNotEmpty(messages)) {
            if (LLMRoleEnum.USER.getRole().equals(messages.get(messages.size() - 1).getRole())) {
                Long recordId = recordModels.get(recordModels.size() - 1).getId();
                bizLongTextDialoguesRecordService.deleteById(recordId);
                messages.remove(messages.size() - 1);
            }
        }

        // 用户输入
        Message message = new Message();
        message.setContent(input);
        message.setRole(LLMRoleEnum.USER.getRole());
        messages.add(message);
        logger.info("--------- Build Messages dialogsId:{},messages:{}--------------", dialogsId, messages);
        return messages;
    }

    /**
     * 文本输出结果
     *
     * @param sseUtil
     * @param bufferedReader
     * @throws IOException
     */
    private LongtextDialoguesResult textOutputStream(SSEUtil sseUtil, BufferedReader bufferedReader) throws IOException {
        String res = "";
        StringBuilder output = new StringBuilder();
        StringBuilder reasoningContent = new StringBuilder();
        try {
            while ((res = bufferedReader.readLine()) != null) {
                if (StringUtils.isEmpty(res)) {
                    continue;
                }
                res = StringUtils.replace(res, "data: ", StringUtils.EMPTY);
                LargeModelDemandResult result = JsonUtils.deSerialize(res, LargeModelDemandResult.class);
                if (!"0".equals(result.getCode())) {
                    logger.error("LLM Error,code:{}", result.getCode());
                    BusinessException ex = new BusinessException("exception/call.failure");
                    result.setMessage(ex.getMessage());
                    sseUtil.send(JsonUtils.serialize(result));
                    sseUtil.send("[DONE]");
                    sseUtil.complete();
                    throw ex;
                }
                result.setFunction(null);
                result.setDbChainResult(null);
                result.setKnowledgeContentResult(null);
                sseUtil.send(JsonUtils.serialize(result));
                if (StringUtils.isNotEmpty(result.getMessage())) {
                    output.append(result.getMessage());
                }
                if (StringUtils.isNotEmpty(result.getReasoningContent())) {
                    reasoningContent.append(result.getReasoningContent());
                }
            }
            // 关闭资源
            sseUtil.send("[DONE]");
            sseUtil.complete();
            LongtextDialoguesResult longtextDialoguesResult = new LongtextDialoguesResult();
            longtextDialoguesResult.setMessage(output.toString());
            longtextDialoguesResult.setReasoningContent(reasoningContent.toString());
            return longtextDialoguesResult;
        } catch (IOException e) {
            logger.error("连接断开,code:{}", e.getMessage());
            sseUtil.completeByError("连接断开");
            throw new BusinessException("连接断开");
        } finally {
            bufferedReader.close();
        }
    }


    private List<KnowledgeContentResult> knowledge(List<Integer> kdIds, String input, Integer topK, Double score, KnowledgeSearchTypeEnum searchTypeEnum) {
        List<KnowledgeContentResult> knowledgeContentResults = new ArrayList<>();
        if (CollectionUtils.isEmpty(kdIds)) {
            return knowledgeContentResults;
        }
        List<String> knowledgeIds = new ArrayList<>();
        for (Integer kdId : kdIds) {
            BizKnowledgeDocumentEntity knowledgeDocumentEntity = bizKnowledgeDocumentService.get(kdId);
            // 筛选训练完成的文档，否则跳过该文档
            if (null == knowledgeDocumentEntity && KnowledgeConstant.TrainStatus.COMPLETE.equals(knowledgeDocumentEntity.getTrainStatus())) {
                continue;
            }
            knowledgeIds.add(knowledgeDocumentEntity.getKnowledgeId());
        }

        SearchKnowledgeResult searchKnowledgeResult = demandKnowledgeService.searchKnowledge(input, knowledgeIds, topK, score, searchTypeEnum);

        if (CollectionUtils.isNotEmpty(searchKnowledgeResult.getDocuments())) {
            for (int i = 1; i <= searchKnowledgeResult.getDocuments().size(); i++) {
                KnowledgeContentResult knowledgeContentResult = new KnowledgeContentResult();
                knowledgeContentResult.setContent(searchKnowledgeResult.getDocuments().get(i - 1));
                knowledgeContentResult.setKnowledgeId(searchKnowledgeResult.getKnowledgeIds().get(i - 1));
                knowledgeContentResult.setScore(searchKnowledgeResult.getScore().get(i - 1));
                knowledgeContentResults.add(knowledgeContentResult);
            }
            // 根据knowledgeId获取知识库名和文档名
            knowledgeIds = knowledgeContentResults.stream().map(KnowledgeContentResult::getKnowledgeId).distinct().collect(Collectors.toList());
            List<KnowledgeDocumentRelationQueryItem> knowledgeDocumentRelationQueryItems = bizKnowledgeDocumentService.knowledgeDocumentRelationQuery(knowledgeIds, null);
            for (KnowledgeContentResult result : knowledgeContentResults) {
                String knowledgeId = result.getKnowledgeId();
                KnowledgeDocumentRelationQueryItem item = knowledgeDocumentRelationQueryItems.stream().filter(v -> v.getKnowledgeId().equals(knowledgeId)).findFirst().get();
                result.setKnowledgeName(item.getKnowledgeName());
                result.setKdId(item.getKdId());
                result.setDocumentName(item.getDocumentName());
            }
        }

        return knowledgeContentResults;
    }
}
